@@ -11,181 +11,42 @@ import matplotlib
 from matplotlib import pyplot as plt
 from matplotlib import colors
 from matplotlib import patches
-from matplotlib.tight_layout import get_renderer
+
+from .reformat import query, _get_subset_mask
+from . import util
+
+# prevents ImportError on matplotlib versions >3.5.2
+try:
+    from matplotlib.tight_layout import get_renderer
+
+    RENDERER_IMPORTED = True
+except ImportError:
+    RENDERER_IMPORTED = False
 
 
-def _aggregate_data(df, subset_size, sum_over):
-    """
-    Returns
-    -------
-    df : DataFrame
-        full data frame
-    aggregated : Series
-        aggregates
-    """
-    _SUBSET_SIZE_VALUES = ['auto', 'count', 'sum']
-    if subset_size not in _SUBSET_SIZE_VALUES:
-        raise ValueError('subset_size should be one of %s. Got %r'
-                         % (_SUBSET_SIZE_VALUES, subset_size))
-    if df.ndim == 1:
-        # Series
-        input_name = df.name
-        df = pd.DataFrame({'_value': df})
-
-        if subset_size == 'auto' and not df.index.is_unique:
-            raise ValueError('subset_size="auto" cannot be used for a '
-                             'Series with non-unique groups.')
-        if sum_over is not None:
-            raise ValueError('sum_over is not applicable when the input is a '
-                             'Series')
-        if subset_size == 'count':
-            sum_over = False
-        else:
-            sum_over = '_value'
-    else:
-        # DataFrame
-        if sum_over is False:
-            raise ValueError('Unsupported value for sum_over: False')
-        elif subset_size == 'auto' and sum_over is None:
-            sum_over = False
-        elif subset_size == 'count':
-            if sum_over is not None:
-                raise ValueError('sum_over cannot be set if subset_size=%r' %
-                                 subset_size)
-            sum_over = False
-        elif subset_size == 'sum':
-            if sum_over is None:
-                raise ValueError('sum_over should be a field name if '
-                                 'subset_size="sum" and a DataFrame is '
-                                 'provided.')
-
-    gb = df.groupby(level=list(range(df.index.nlevels)), sort=False)
-    if sum_over is False:
-        aggregated = gb.size()
-        aggregated.name = 'size'
-    elif hasattr(sum_over, 'lower'):
-        aggregated = gb[sum_over].sum()
-    else:
-        raise ValueError('Unsupported value for sum_over: %r' % sum_over)
-
-    if aggregated.name == '_value':
-        aggregated.name = input_name
-
-    return df, aggregated
-
-
-def _check_index(df):
-    # check all indices are boolean
-    if not all(set([True, False]) >= set(level)
-               for level in df.index.levels):
-        raise ValueError('The DataFrame has values in its index that are not '
-                         'boolean')
-    df = df.copy(deep=False)
-    # XXX: this may break if input is not MultiIndex
-    kw = {'levels': [x.astype(bool) for x in df.index.levels],
-          'names': df.index.names,
-          }
-    if hasattr(df.index, 'codes'):
-        # compat for pandas <= 0.20
-        kw['codes'] = df.index.codes
-    else:
-        kw['labels'] = df.index.labels
-    df.index = pd.MultiIndex(**kw)
-    return df
-
-
-def _scalar_to_list(val):
-    if not isinstance(val, (typing.Sequence, set)) or isinstance(val, str):
-        val = [val]
-    return val
-
-
-def _get_subset_mask(agg, min_subset_size, max_subset_size,
-                     min_degree, max_degree,
-                     present, absent):
-    """Get a mask over subsets based on size, degree or category presence"""
-    subset_mask = True
-    if min_subset_size is not None:
-        subset_mask = np.logical_and(subset_mask, agg >= min_subset_size)
-    if max_subset_size is not None:
-        subset_mask = np.logical_and(subset_mask, agg <= max_subset_size)
-    if (min_degree is not None and min_degree >= 0) or max_degree is not None:
-        degree = agg.index.to_frame().sum(axis=1)
-        if min_degree is not None:
-            subset_mask = np.logical_and(subset_mask, degree >= min_degree)
-        if max_degree is not None:
-            subset_mask = np.logical_and(subset_mask, degree <= max_degree)
-    if present is not None:
-        for col in _scalar_to_list(present):
-            subset_mask = np.logical_and(
-                subset_mask,
-                agg.index.get_level_values(col).values)
-    if absent is not None:
-        for col in _scalar_to_list(absent):
-            exclude_mask = np.logical_not(
-                agg.index.get_level_values(col).values)
-            subset_mask = np.logical_and(subset_mask, exclude_mask)
-    return subset_mask
-
-
-def _filter_subsets(df, agg,
-                    min_subset_size, max_subset_size,
-                    min_degree, max_degree):
-    subset_mask = _get_subset_mask(agg,
-                                   min_subset_size=min_subset_size,
-                                   max_subset_size=max_subset_size,
-                                   min_degree=min_degree,
-                                   max_degree=max_degree,
-                                   present=None, absent=None)
-
-    if subset_mask is True:
-        return df, agg
-
-    agg = agg[subset_mask]
-    df = df[df.index.isin(agg.index)]
-    return df, agg
-
-
-def _process_data(df, sort_by, sort_categories_by, subset_size,
+def _process_data(df, *, sort_by, sort_categories_by, subset_size,
                   sum_over, min_subset_size=None, max_subset_size=None,
-                  min_degree=None, max_degree=None, reverse=False):
-    df, agg = _aggregate_data(df, subset_size, sum_over)
+                  min_degree=None, max_degree=None, reverse=False,
+                  include_empty_subsets=False):
+    results = query(df, sort_by=sort_by, sort_categories_by=sort_categories_by,
+                    subset_size=subset_size, sum_over=sum_over,
+                    min_subset_size=min_subset_size,
+                    max_subset_size=max_subset_size,
+                    min_degree=min_degree, max_degree=max_degree,
+                    include_empty_subsets=include_empty_subsets)
+
+    df = results.data
+    agg = results.subset_sizes
+    totals = results.category_totals
     total = agg.sum()
-    df = _check_index(df)
-    totals = [agg[agg.index.get_level_values(name).values.astype(bool)].sum()
-              for name in agg.index.names]
-    totals = pd.Series(totals, index=agg.index.names)
-
-    # filter subsets:
-    df, agg = _filter_subsets(df, agg,
-                              min_subset_size, max_subset_size,
-                              min_degree, max_degree)
-
-    # sort:
-    if sort_categories_by == 'cardinality':
-        totals.sort_values(ascending=False, inplace=True)
-    elif sort_categories_by is not None:
-        raise ValueError('Unknown sort_categories_by: %r' % sort_categories_by)
-    df = df.reorder_levels(totals.index.values)
-    agg = agg.reorder_levels(totals.index.values)
-
-    if sort_by == 'cardinality':
-        agg = agg.sort_values(ascending=False)
-    elif sort_by == 'degree':
-        index_tuples = sorted(agg.index,
-                              key=lambda x: (sum(x),) + tuple(reversed(x)))
-        agg = agg.reindex(pd.MultiIndex.from_tuples(index_tuples,
-                                                    names=agg.index.names))
-    elif sort_by is None:
-        pass
-    else:
-        raise ValueError('Unknown sort_by: %r' % sort_by)
 
     # add '_bin' to df indicating index in agg
     # XXX: ugly!
     def _pack_binary(X):
         X = pd.DataFrame(X)
-        out = 0
+        # use objects if arbitrary precision integers are needed
+        dtype = np.object_ if X.shape[1] > 62 else np.uint64
+        out = pd.Series(0, index=X.index, dtype=dtype)
         for i, (_, col) in enumerate(X.items()):
             out *= 2
             out += col
@@ -198,6 +59,7 @@ def _process_data(df, sort_by, sort_categories_by, subset_size,
                   index=data_packed))
     if reverse:
         agg = agg[::-1]
+
     return total, df, agg, totals
 
 
@@ -284,19 +146,19 @@ class UpSet:
         If a DataFrame, `sum_over` must be a string or False.
     orientation : {'horizontal' (default), 'vertical'}
         If horizontal, intersections are listed from left to right.
-    sort_by : {'cardinality', 'degree', None}
+    sort_by : {'cardinality', 'degree', '-cardinality', '-degree',
+               'input', '-input'}
         If 'cardinality', subset are listed from largest to smallest.
         If 'degree', they are listed in order of the number of categories
-        intersected. If None, the order they appear in the data input is
+        intersected. If 'input', the order they appear in the data input is
         used.
+        Prefix with '-' to reverse the ordering.
 
-        .. versionchanged: 0.5
-            Setting None was added.
-    sort_categories_by : {'cardinality', None}
+        Note this affects ``subset_sizes`` but not ``data``.
+    sort_categories_by : {'cardinality', '-cardinality', 'input', '-input'}
         Whether to sort the categories by total cardinality, or leave them
-        in the provided order.
-
-        .. versionadded: 0.3
+        in the input data's provided order (order of index levels).
+        Prefix with '-' to reverse the ordering.
     subset_size : {'auto', 'count', 'sum'}
         Configures how to calculate the size of a subset. Choices are:
 
@@ -320,36 +182,36 @@ class UpSet:
         a size smaller than this threshold will be omitted from plotting.
         Size may be a sum of values, see `subset_size`.
 
-        .. versionadded: 0.5
+        .. versionadded:: 0.5
     max_subset_size : int, optional
         Maximum size of a subset to be shown in the plot. All subsets with
         a size greater than this threshold will be omitted from plotting.
 
-        .. versionadded: 0.5
+        .. versionadded:: 0.5
     min_degree : int, optional
         Minimum degree of a subset to be shown in the plot.
 
-        .. versionadded: 0.5
+        .. versionadded:: 0.5
     max_degree : int, optional
         Maximum degree of a subset to be shown in the plot.
 
-        .. versionadded: 0.5
+        .. versionadded:: 0.5
     facecolor : 'auto' or matplotlib color or float
         Color for bar charts and active dots. Defaults to black if
         axes.facecolor is a light color, otherwise white.
 
-        .. versionchanged: 0.6
+        .. versionchanged:: 0.6
             Before 0.6, the default was 'black'
     other_dots_color : matplotlib color or float
         Color for shading of inactive dots, or opacity (between 0 and 1)
         applied to facecolor.
 
-        .. versionadded: 0.6
+        .. versionadded:: 0.6
     shading_color : matplotlib color or float
         Color for shading of odd rows in matrix and totals, or opacity (between
         0 and 1) applied to facecolor.
 
-        .. versionadded: 0.6
+        .. versionadded:: 0.6
     with_lines : bool
         Whether to show lines joining dots in the matrix, to mark multiple
         categories being intersected.
@@ -359,7 +221,7 @@ class UpSet:
         The intersections plot should be large enough to fit this many matrix
         elements. Set to 0 to disable intersection size bars.
 
-        .. versionchanged: 0.4
+        .. versionchanged:: 0.4
             Setting to 0 is handled.
     totals_plot_elements : int
         The totals plot should be large enough to fit this many matrix
@@ -367,13 +229,21 @@ class UpSet:
     show_counts : bool or str, default=False
         Whether to label the intersection size bars with the cardinality
         of the intersection. When a string, this formats the number.
-        For example, '%d' is equivalent to True.
-    show_percentages : bool, default=False
+        For example, '{:d}' is equivalent to True.
+        Note that, for legacy reasons, if the string does not contain '{',
+        it will be interpreted as a C-style format string, such as '%d'.
+    show_percentages : bool or str, default=False
         Whether to label the intersection size bars with the percentage
         of the intersection relative to the total dataset.
+        When a string, this formats the number representing a fraction of
+        samples.
+        For example, '{:.1%}' is the default, formatting .123 as 12.3%.
         This may be applied with or without show_counts.
 
-        .. versionadded: 0.4
+        .. versionadded:: 0.4
+    include_empty_subsets : bool (default=False)
+        If True, all possible category combinations will be shown as subsets,
+        even when some are not present in data.
     """
     _default_figsize = (10, 6)
 
@@ -385,7 +255,8 @@ class UpSet:
                  facecolor='auto', other_dots_color=.18, shading_color=.05,
                  with_lines=True, element_size=32,
                  intersection_plot_elements=6, totals_plot_elements=2,
-                 show_counts='', show_percentages=False):
+                 show_counts='', show_percentages=False,
+                 include_empty_subsets=False):
 
         self._horizontal = orientation == 'horizontal'
         self._reorient = _identity if self._horizontal else _transpose
@@ -413,16 +284,18 @@ class UpSet:
         self._show_percentages = show_percentages
 
         (self.total, self._df, self.intersections,
-         self.totals) = _process_data(data,
-                                      sort_by=sort_by,
-                                      sort_categories_by=sort_categories_by,
-                                      subset_size=subset_size,
-                                      sum_over=sum_over,
-                                      min_subset_size=min_subset_size,
-                                      max_subset_size=max_subset_size,
-                                      min_degree=min_degree,
-                                      max_degree=max_degree,
-                                      reverse=not self._horizontal)
+         self.totals) = _process_data(
+            data,
+            sort_by=sort_by,
+            sort_categories_by=sort_categories_by,
+            subset_size=subset_size,
+            sum_over=sum_over,
+            min_subset_size=min_subset_size,
+            max_subset_size=max_subset_size,
+            min_degree=min_degree,
+            max_degree=max_degree,
+            reverse=not self._horizontal,
+            include_empty_subsets=include_empty_subsets)
         self.subset_styles = [{"facecolor": facecolor}
                               for i in range(len(self.intersections))]
         self.subset_legend = []  # pairs of (style, label)
@@ -570,7 +443,7 @@ class UpSet:
 
         Used to plot categorical variable distributions within each subset.
 
-        .. versionadded: 0.6
+        .. versionadded:: 0.6
 
         Parameters
         ----------
@@ -697,16 +570,22 @@ class UpSet:
             fig = plt.gcf()
 
         # Determine text size to determine figure size / spacing
-        r = get_renderer(fig)
         text_kw = {"size": matplotlib.rcParams['xtick.labelsize']}
         # adding "x" ensures a margin
         t = fig.text(0, 0, '\n'.join(str(label) + "x"
                                      for label in self.totals.index.values),
                      **text_kw)
-        textw = t.get_window_extent(renderer=r).width
+        window_extent_args = {}
+        if RENDERER_IMPORTED:
+            window_extent_args["renderer"] = get_renderer(fig)
+        textw = t.get_window_extent(**window_extent_args).width
         t.remove()
 
-        figw = self._reorient(fig.get_window_extent(renderer=r)).width
+        window_extent_args = {}
+        if RENDERER_IMPORTED:
+            window_extent_args["renderer"] = get_renderer(fig)
+        figw = self._reorient(
+            fig.get_window_extent(**window_extent_args)).width
 
         sizes = np.asarray([p['elements'] for p in self._subset_plots])
         fig = self._reorient(fig)
@@ -840,18 +719,21 @@ class UpSet:
 
         if self.subset_legend:
             styles, labels = zip(*self.subset_legend)
-            styles = [patches.Patch(**style) for style in styles]
+            styles = [patches.Patch(**patch_style) for patch_style in styles]
             ax.legend(styles, labels)
 
     def _label_sizes(self, ax, rects, where):
         if not self._show_counts and not self._show_percentages:
             return
         if self._show_counts is True:
-            count_fmt = "%d"
+            count_fmt = "{:.0f}"
         else:
             count_fmt = self._show_counts
+            if '{' not in count_fmt:
+                count_fmt = util.to_new_pos_format(count_fmt)
+
         if self._show_percentages is True:
-            pct_fmt = "%.1f%%"
+            pct_fmt = "{:.1%}"
         else:
             pct_fmt = self._show_percentages
 
@@ -862,7 +744,7 @@ class UpSet:
                 fmt = '%s (%s)' % (count_fmt, pct_fmt)
 
             def make_args(val):
-                return val, 100 * val / self.total
+                return val, val / self.total
         elif count_fmt:
             fmt = count_fmt
 
@@ -872,7 +754,7 @@ class UpSet:
             fmt = pct_fmt
 
             def make_args(val):
-                return 100 * val / self.total,
+                return val / self.total,
 
         if where == 'right':
             margin = 0.01 * abs(np.diff(ax.get_xlim()))
@@ -880,7 +762,7 @@ class UpSet:
                 width = rect.get_width() + rect.get_x()
                 ax.text(width + margin,
                         rect.get_y() + rect.get_height() * .5,
-                        fmt % make_args(width),
+                        fmt.format(*make_args(width)),
                         ha='left', va='center')
         elif where == 'left':
             margin = 0.01 * abs(np.diff(ax.get_xlim()))
@@ -888,7 +770,7 @@ class UpSet:
                 width = rect.get_width() + rect.get_x()
                 ax.text(width + margin,
                         rect.get_y() + rect.get_height() * .5,
-                        fmt % make_args(width),
+                        fmt.format(*make_args(width)),
                         ha='right', va='center')
         elif where == 'top':
             margin = 0.01 * abs(np.diff(ax.get_ylim()))
@@ -896,7 +778,7 @@ class UpSet:
                 height = rect.get_height() + rect.get_y()
                 ax.text(rect.get_x() + rect.get_width() * .5,
                         height + margin,
-                        fmt % make_args(height),
+                        fmt.format(*make_args(height)),
                         ha='center', va='bottom')
         else:
             raise NotImplementedError('unhandled where: %r' % where)
